#! /usr/bin/env python
# -*- coding: utf-8 -*-
####################
# Control an Aviosys IP9528 IP PDU from Indigo
# V1.0	28 October 2012
# Copyright (c) 2012, Nick Smith. All rights reserved
#
# Based on example code from Indigo SDK v1.02
# Copyright (c) 2012, Perceptive Automation, LLC. All rights reserved.
# http://www.perceptiveautomation.com

import os
import sys
import urllib2
import socket
import string

# Note the "indigo" module is automatically imported and made available inside
# our global name space by the host process.

################################################################################
class Plugin(indigo.PluginBase):

	REALM = 'IP9258'
	auth_handler = urllib2.HTTPBasicAuthHandler()
	
	########################################
	def __init__(self, pluginId, pluginDisplayName, pluginVersion, pluginPrefs):
		indigo.PluginBase.__init__(self, pluginId, pluginDisplayName, pluginVersion, pluginPrefs)
		self.debug = pluginPrefs.get("showDebugInfo", False)
		
	def __del__(self):
		indigo.PluginBase.__del__(self)

	########################################
	def startup(self):
		self.debugLog(u"startup called")
		# setup a socket timeout
		timeout = 5
		socket.setdefaulttimeout(timeout)
		
	def shutdown(self):
		self.debugLog(u"shutdown called")
		
	########################################
	def deviceStartComm(self, dev):
		self.debugLog("Starting device: " + dev.name)
		
		# get username & password from props & add to opener
		userName = dev.pluginProps["userName"]
		password = dev.pluginProps["password"]
		pduIpAddr = dev.pluginProps["ipAddr"]
		outlet = dev.pluginProps["outlet"]
		
		self.debugLog("Username: " + userName)
		self.debugLog("Password: " + password)
		self.debugLog("IP address: " + pduIpAddr)
		self.debugLog("Outlet: " + outlet)
		
		# Create an OpenerDirector to handle basic http authentication
   		#self.auth_handler = urllib2.HTTPBasicAuthHandler()
		self.auth_handler.add_password(self.REALM, pduIpAddr, userName, password)
		opener = urllib2.build_opener(self.auth_handler)
		urllib2.install_opener(opener)
		
		# read real state of port & set internal state variable
		self.readAndUpdateState(dev)

	########################################
	def validateDeviceConfigUi(self, valuesDict, typeId, devId):
		# validate supplied values
		outletNum = int(valuesDict["outlet"])
		if outletNum < 1 or outletNum > 4:
			self.errorLog(u"Error: Outlet \"%s\" must be between 1 & 4" % str(outletNum))
			errorDict = indigo.Dict()
			errorDict["outlet"] = "The value of this field must be between 1 & 4"
			return (False, valuesDict, errorDict)
		else:
			return True

	########################################
	# Relay / Dimmer Action callback
	######################
	def actionControlDimmerRelay(self, action, dev):
			
		###### TURN ON ######
		if action.deviceAction == indigo.kDeviceAction.TurnOn:
			# Command hardware module (dev) to turn ON here:						
			if self.setPDUState(dev, "on") == 0:
				sendSuccess = True
			else:
				sendSuccess = False

			if sendSuccess:
				# If success then log that the command was successfully sent.
				indigo.server.log(u"Turned \"%s\" %s" % (dev.name, "on"))
	
				# And then tell the Indigo Server to update the state.
				dev.updateStateOnServer("onOffState", True)
			else:
				# Else log failure but do NOT update state on Indigo Server.
				indigo.server.log(u"send \"%s\" %s failed" % (dev.name, "on"), isError=True)

		###### TURN OFF ######
		elif action.deviceAction == indigo.kDeviceAction.TurnOff:
			# Command hardware module (dev) to turn OFF here:			
			if self.setPDUState(dev, "off") == 0:
				sendSuccess = True
			else:
				sendSuccess = False

			if sendSuccess:
				# If success then log that the command was successfully sent.
				indigo.server.log(u"Turned \"%s\" %s" % (dev.name, "off"))
	
				# And then tell the Indigo Server to update the state:
				dev.updateStateOnServer("onOffState", False)
			else:
				# Else log failure but do NOT update state on Indigo Server.
				indigo.server.log(u"send \"%s\" %s failed" % (dev.name, "off"), isError=True)

		###### TOGGLE ######
		elif action.deviceAction == indigo.kDeviceAction.Toggle:
			# Command hardware module (dev) to toggle here:
			sendSuccess = False
			newOnState = not dev.onState
			if newOnState == True:
				# currently off, so turn on
				if self.setPDUState(dev, "on") == 0:
					sendSuccess = True
			else:
				# currently on, so turn off
				if self.setPDUState(dev, "off") == 0:
					sendSuccess = True
				
			if sendSuccess:
				# If success then log that the command was successfully sent.
				indigo.server.log(u"sent \"%s\" %s" % (dev.name, "toggle"))
	
				# And then tell the Indigo Server to update the state:
				dev.updateStateOnServer("onOffState", newOnState)
			else:
				# Else log failure but do NOT update state on Indigo Server.
				indigo.server.log(u"send \"%s\" %s failed" % (dev.name, "toggle"), isError=True)

		###### STATUS REQUEST ######
		elif action.deviceAction == indigo.kDeviceAction.RequestStatus:
			# Query hardware module (dev) for its current states here:
			self.readAndUpdateState(dev)
			

	def setPDUState(self, dev, state):
		# send command to PDU to change state of an outlet
		# state argument is either "on" or "off"
		
		# validate inputs
		state_num = 2
		if string.lower(state) == "on":
			state_num = 1
		elif string.lower(state) == "off":
			state_num = 0
		else:
			self.errorLog(u"Error: State must be on or off")
			return(state_num)
			
		# get IP & outlet from props
		pduIpAddr = dev.pluginProps["ipAddr"]
		outlet = dev.pluginProps["outlet"]
	
		# build the string to send to the PDU
		url_cmd = "http://" + pduIpAddr + "/set.cmd?cmd=setpower+p6" + outlet + "=" + str(state_num)
		self.debugLog(u"Sending to PDU: " + url_cmd)
	
		try:
			# send the command to the PDU & clean up afterwards
			response = urllib2.urlopen(url_cmd)
			response.read()
			response.close()
		
		except urllib2.URLError, e:
			if hasattr(e, 'reason'):
				self.errorLog(u"Error: We failed to reach a server.")
				self.errorLog("Reason: " + str(e.reason))
				return(2)
			elif hasattr(e, 'code'):
				self.errorLog(u"Error: The server couldn\'t fulfill the request.")
				self.errorLog("Error code: " + str(e.code))
				return(2)
		
		else:
			# everything worked
			self.debugLog(u"Sent to PDU")
			return(0)
			
	def getPDUState(self, dev):
		# request state of PDU
		# returns state of all outlets in a single string
		
		# get IP & outlet from props
		pduIpAddr = dev.pluginProps["ipAddr"]
		outlet = dev.pluginProps["outlet"]
		
		# build the request string to send to the PDU
		url_cmd = "http://" + pduIpAddr + "/set.cmd?cmd=getpower"
	
		resultCode = 0
		
		try:
			# send the command to the PDU & clean up afterwards
			response = urllib2.urlopen(url_cmd)
			resultString = response.read()
			response.close()
		
		except urllib2.URLError, e:
			if hasattr(e, 'reason'):
				self.errorLog(u"Error: We failed to reach a server.")
				self.errorLog("Reason: " + str(e.reason))
				return(2)
			elif hasattr(e, 'code'):
				self.errorLog(u"Error: The server couldn\'t fulfill the request.")
				self.errorLog("Error code: " + str(e.code))
				return(2)
		
		else:
			# everything worked
			self.debugLog(u"Sent to PDU: " + url_cmd)
			self.debugLog(u"Received from PDU: " + resultString)

			
			# check if this outlet status is in string returned from PDU
			# if so, grab the 5th char which will be either 1 (on) or 0 (off)
			outletString = "p6" + outlet + "="
			index = resultString.find(outletString)
			if index != -1:
				resultCode = int(resultString[index+4])
				self.debugLog(u"Received code " + str(resultCode))
			else:
				self.errorLog(u"Error: received unexpected response \"" + resultString + "\"")
				resultCode = 2
			
			return(resultCode)
			
	
			
	def readAndUpdateState(self, dev):
		# request state from the PDU & update state variable accordingly
		resultCode = self.getPDUState(dev)
		
		if resultCode == 0:
			dev.updateStateOnServer("onOffState", False)
			indigo.server.log(u"Device %s is off" % dev.name)
			return(True)
		elif resultCode == 1:
			dev.updateStateOnServer("onOffState", True)
			indigo.server.log(u"Device %s is on" % dev.name)
			return(True)
		else:
			self.errorLog(u"Error: PDU %s in unknown state" % dev.name)
			return(False)
			
		
		